@@ -8,7 +8,7 @@
  *
  *    In some countries OX, OX Open-Xchange, open xchange and OXtender
  *    as well as the corresponding Logos OX Open-Xchange and OX are registered
- *    trademarks of the Open-Xchange, Inc. group of companies.
+ *    trademarks of the OX Software GmbH group of companies.
  *    The use of the Logos is not covered by the GNU General Public License.
  *    Instead, you are allowed to use these Logos according to the terms and
  *    conditions of the Creative Commons License, Version 2.5, Attribution,
@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2013 Open-Xchange, Inc.
+ *     Copyright (C) 2016-2020 OX Software GmbH
  *     Mail: info@open-xchange.com
  *
  *
@@ -49,8 +49,6 @@
 
 package com.openexchange.spamhandler.cloudmark;
 
-import static com.openexchange.spamhandler.cloudmark.osgi.CloudmarkSpamHandlerServiceRegistry.getServiceRegistry;
-import javax.mail.Address;
 import javax.mail.internet.AddressException;
 import javax.mail.internet.InternetAddress;
 import com.openexchange.config.ConfigurationService;
@@ -59,6 +57,10 @@ import com.openexchange.mail.MailField;
 import com.openexchange.mail.api.MailAccess;
 import com.openexchange.mail.dataobjects.MailMessage;
 import com.openexchange.mail.transport.MailTransport;
+import com.openexchange.mail.transport.MailTransport.SendRawProperties;
+import com.openexchange.mail.usersetting.UserSettingMail;
+import com.openexchange.mail.usersetting.UserSettingMailStorage;
+import com.openexchange.server.ServiceLookup;
 import com.openexchange.session.Session;
 import com.openexchange.spamhandler.SpamHandler;
 
@@ -66,6 +68,7 @@ import com.openexchange.spamhandler.SpamHandler;
  * Cloudmark spam handler
  *
  * @author <a href="mailto:benjamin.otterbach@open-xchange.com">Benjamin Otterbach</a>
+ * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
  */
 public final class CloudmarkSpamHandler extends SpamHandler {
 
@@ -73,14 +76,16 @@ public final class CloudmarkSpamHandler extends SpamHandler {
 
     private static final String NAME = "CloudmarkSpamHandler";
 
-    private static final CloudmarkSpamHandler SINGLETON = new CloudmarkSpamHandler();
+    // -------------------------------------------------------------------------------------------
 
-    public static CloudmarkSpamHandler getInstance() {
-        return SINGLETON;
-    }
+    private final ServiceLookup services;
 
-    private CloudmarkSpamHandler() {
+    /**
+     * Initializes a new {@link CloudmarkSpamHandler}.
+     */
+    public CloudmarkSpamHandler(ServiceLookup services) {
         super();
+        this.services = services;
     }
 
     @Override
@@ -90,7 +95,7 @@ public final class CloudmarkSpamHandler extends SpamHandler {
 
     @Override
     public void handleSpam(final int accountId, final String fullName, final String[] mailIDs, final boolean move, final Session session) throws OXException {
-        final ConfigurationService configuration = getServiceRegistry().getService(ConfigurationService.class);
+        final ConfigurationService configuration = services.getService(ConfigurationService.class);
         final String targetSpamEmailAddress = configuration.getProperty("com.openexchange.spamhandler.cloudmark.targetSpamEmailAddress", "").trim();
 
         MailAccess<?, ?> mailAccess = null;
@@ -102,10 +107,15 @@ public final class CloudmarkSpamHandler extends SpamHandler {
                 final MailTransport transport = MailTransport.getInstance(session);
                 try {
                     if (isEmpty(targetSpamEmailAddress)) {
-                        LOG.error("There is no value configured for 'com.openexchange.spamhandler.cloudmark.targetSpamEmailAddress', cannot process spam reporting to server.");
-                        return;
+                        LOG.debug("There is no value configured for 'com.openexchange.spamhandler.cloudmark.targetSpamEmailAddress', cannot process spam reporting to server.");
+                    } else {
+                        SendRawProperties sendRawProperties = MailTransport.SendRawProperties.newInstance()
+                            .addRecipient(new InternetAddress(targetSpamEmailAddress, true))
+                            .setSender(getSenderAddress(session))
+                            .setValidateAddressHeaders(false)
+                            .setSanitizeHeaders(false);
+                        transport.sendRawMessage(mailMessage[i].getSourceBytes(), sendRawProperties);
                     }
-                    transport.sendRawMessage(mailMessage[i].getSourceBytes(), new Address[] { new InternetAddress(targetSpamEmailAddress, true) });
                 } catch (final AddressException e) {
                     LOG.error("The configured target eMail address is not valid", e);
                 } finally {
@@ -115,7 +125,6 @@ public final class CloudmarkSpamHandler extends SpamHandler {
 
             if (move) {
                 final String targetSpamFolder = configuration.getProperty("com.openexchange.spamhandler.cloudmark.targetSpamFolder", "1").trim();
-
                 if (targetSpamFolder.equals("1")) {
                     mailAccess.getMessageStorage().moveMessages(fullName, mailAccess.getFolderStorage().getTrashFolder(), mailIDs, true);
                 } else if (targetSpamFolder.equals("2")) {
@@ -136,6 +145,61 @@ public final class CloudmarkSpamHandler extends SpamHandler {
         }
     }
 
+    @Override
+    public void handleHam(final int accountId, final String fullname, final String[] mailIDs, final boolean move, final Session session) throws OXException {
+        ConfigurationService configuration = services.getService(ConfigurationService.class);
+        String targetHamEmailAddress = configuration.getProperty("com.openexchange.spamhandler.cloudmark.targetHamEmailAddress", "").trim();
+
+        MailAccess<?, ?> mailAccess = null;
+        try {
+            mailAccess = MailAccess.getInstance(session, accountId);
+            mailAccess.connect();
+
+            MailMessage[] mailMessage = mailAccess.getMessageStorage().getMessages(fullname, mailIDs, new MailField[]{MailField.FULL});
+            for (int i = 0; i < mailMessage.length; i++) {
+                final MailTransport transport = MailTransport.getInstance(session);
+                try {
+                    if (isEmpty(targetHamEmailAddress)) {
+                        LOG.debug("There is no value configured for 'com.openexchange.spamhandler.cloudmark.targetHamEmailAddress', cannot process ham reporting to server.");
+                    } else {
+                        SendRawProperties sendRawProperties = MailTransport.SendRawProperties.newInstance()
+                            .addRecipient(new InternetAddress(targetHamEmailAddress, true))
+                            .setSender(getSenderAddress(session))
+                            .setValidateAddressHeaders(false)
+                            .setSanitizeHeaders(false);
+                        transport.sendRawMessage(mailMessage[i].getSourceBytes(), sendRawProperties);
+                    }
+                } catch (final AddressException e) {
+                    LOG.error("The configured target eMail address is not valid", e);
+                } finally {
+                    transport.close();
+                }
+            }
+
+            if (move) {
+                String targetSpamFolder = configuration.getProperty("com.openexchange.spamhandler.cloudmark.targetSpamFolder", "1").trim();
+                if (!targetSpamFolder.equals("0")) {
+                    mailAccess.getMessageStorage().moveMessages(fullname, "INBOX", mailIDs, true);
+                }
+            }
+        } finally {
+            if (null != mailAccess) {
+                mailAccess.close(true);
+            }
+        }
+    }
+
+    @Override
+    public boolean isCreateConfirmedSpam() {
+        final ConfigurationService configuration = services.getService(ConfigurationService.class);
+        return configuration.getProperty("com.openexchange.spamhandler.cloudmark.targetSpamFolder", "1").trim().equals("3");
+    }
+
+    @Override
+    public boolean isCreateConfirmedHam() {
+        return false;
+    }
+
     private static boolean isEmpty(final String string) {
         if (null == string) {
             return true;
@@ -148,63 +212,26 @@ public final class CloudmarkSpamHandler extends SpamHandler {
         return isWhitespace;
     }
 
-    @Override
-    public void handleHam(final int accountId, final String fullname, final String[] mailIDs, final boolean move, final Session session) throws OXException {
+    /**
+     * Gets the session users sender address.
+     *
+     * @return The address or <code>null</code> if not configured
+     */
+    private static InternetAddress getSenderAddress(Session session) throws OXException {
+        UserSettingMail usm = UserSettingMailStorage.getInstance().getUserSettingMail(session);
+        if (usm == null) {
+            return null;
+        }
 
-    	final ConfigurationService configuration = getServiceRegistry().getService(ConfigurationService.class);
-        final String targetHamEmailAddress = configuration.getProperty("com.openexchange.spamhandler.cloudmark.targetHamEmailAddress", "").trim();
+        String sendAddr = usm.getSendAddr();
+        if (sendAddr == null) {
+            return null;
+        }
 
-        MailAccess<?, ?> mailAccess = null;
         try {
-            mailAccess = MailAccess.getInstance(session, accountId);
-            mailAccess.connect();
-            final MailMessage[] mailMessage = mailAccess.getMessageStorage().getMessages(fullname, mailIDs, new MailField[]{MailField.FULL});
-            for (int i = 0; i < mailMessage.length; i++) {
-                final MailTransport transport = MailTransport.getInstance(session);
-                try {
-                    if (isEmpty(targetHamEmailAddress)) {
-                        LOG.error("There is no value configured for 'com.openexchange.spamhandler.cloudmark.targetHamEmailAddress', cannot process ham reporting to server.");
-                        return;
-                    }
-                    transport.sendRawMessage(mailMessage[i].getSourceBytes(), new Address[] { new InternetAddress(targetHamEmailAddress, true) });
-                } catch (final AddressException e) {
-                    LOG.error("The configured target eMail address is not valid", e);
-                } finally {
-                    transport.close();
-                }
-            }
-
-            if (move) {
-            	final String targetSpamFolder = configuration.getProperty("com.openexchange.spamhandler.cloudmark.targetSpamFolder", "1").trim();
-
-                if (!targetSpamFolder.equals("0")) {
-                	try {
-                		mailAccess = MailAccess.getInstance(session, accountId);
-                		mailAccess.connect();
-                		mailAccess.getMessageStorage().moveMessages(fullname, "INBOX", mailIDs, true);
-                	} finally {
-                		if (null != mailAccess) {
-                			mailAccess.close(true);
-                		}
-                	}
-                }
-            }
-        } finally {
-            if (null != mailAccess) {
-                mailAccess.close(true);
-            }
+            return new InternetAddress(sendAddr, true);
+        } catch (AddressException e) {
+            return null;
         }
     }
-
-    @Override
-    public boolean isCreateConfirmedSpam() {
-        final ConfigurationService configuration = getServiceRegistry().getService(ConfigurationService.class);
-        final String targetSpamFolder = configuration.getProperty("com.openexchange.spamhandler.cloudmark.targetSpamFolder", "1").trim();
-        return targetSpamFolder.equals("3");
-    }
-
-    @Override
-    public boolean isCreateConfirmedHam() {
-        return false;
-    }
 }
